@@ -7,9 +7,11 @@ use syn::{
 };
 
 use super::{
+    arguments::ArgumentsInfo,
     extract_argument_attrs, extract_default_return_type, extract_defaults, extract_fixtures,
-    extract_partials_return_type, parse_vector_trailing_till_double_comma, Attributes,
-    ExtendWithFunctionAttrs, Fixture,
+    extract_partials_return_type,
+    future::{extract_futures, extract_global_awt},
+    parse_vector_trailing_till_double_comma, Attributes, ExtendWithFunctionAttrs, Fixture,
 };
 use crate::{
     error::ErrorsVec,
@@ -25,6 +27,7 @@ use quote::{format_ident, ToTokens};
 pub(crate) struct FixtureInfo {
     pub(crate) data: FixtureData,
     pub(crate) attributes: FixtureModifiers,
+    pub(crate) arguments: ArgumentsInfo,
 }
 
 impl Parse for FixtureModifiers {
@@ -44,6 +47,7 @@ impl Parse for FixtureInfo {
                     .parse::<Token![::]>()
                     .or_else(|_| Ok(Default::default()))
                     .and_then(|_| input.parse())?,
+                arguments: Default::default(),
             }
         })
     }
@@ -59,13 +63,17 @@ impl ExtendWithFunctionAttrs for FixtureInfo {
             defaults,
             default_return_type,
             partials_return_type,
-            once
+            once,
+            futures,
+            global_awt
         ) = merge_errors!(
             extract_fixtures(item_fn),
             extract_defaults(item_fn),
             extract_default_return_type(item_fn),
             extract_partials_return_type(item_fn),
-            extract_once(item_fn)
+            extract_once(item_fn),
+            extract_futures(item_fn),
+            extract_global_awt(item_fn)
         )?;
         self.data.items.extend(
             fixtures
@@ -79,9 +87,9 @@ impl ExtendWithFunctionAttrs for FixtureInfo {
         for (id, return_type) in partials_return_type {
             self.attributes.set_partial_return_type(id, return_type);
         }
-        if let Some(ident) = once {
-            self.attributes.set_once(ident)
-        };
+        self.arguments.set_once(once);
+        self.arguments.set_global_await(global_awt);
+        self.arguments.set_futures(futures.into_iter());
         Ok(())
     }
 }
@@ -104,8 +112,7 @@ fn parse_attribute_args_just_once<'a, T: Parse>(
                 errors.push(syn::Error::new_spanned(
                     a,
                     format!(
-                        "You cannot use '{}' attribute more than once for the same argument",
-                        name
+                        "You cannot use '{name}' attribute more than once for the same argument"
                     ),
                 ));
                 first
@@ -121,24 +128,27 @@ pub(crate) struct FixturesFunctionExtractor(pub(crate) Vec<Fixture>, pub(crate)
 
 impl VisitMut for FixturesFunctionExtractor {
     fn visit_fn_arg_mut(&mut self, node: &mut FnArg) {
-        if let FnArg::Typed(ref mut arg) = node {
-            let name = match arg.pat.as_ref() {
-                syn::Pat::Ident(ident) => ident.ident.clone(),
-                _ => return,
-            };
-            let (extracted, remain): (Vec<_>, Vec<_>) = std::mem::take(&mut arg.attrs)
-                .into_iter()
-                .partition(|attr| attr_in(attr, &["with", "from"]));
-            arg.attrs = remain;
+        let arg = if let FnArg::Typed(ref mut arg) = node {
+            arg
+        } else {
+            return;
+        };
+        let name = match arg.maybe_ident().cloned() {
+            Some(ident) => ident,
+            _ => return,
+        };
+        let (extracted, remain): (Vec<_>, Vec<_>) = std::mem::take(&mut arg.attrs)
+            .into_iter()
+            .partition(|attr| attr_in(attr, &["with", "from"]));
+        arg.attrs = remain;
 
-            let (pos, errors) = parse_attribute_args_just_once(extracted.iter(), "with");
-            self.1.extend(errors.into_iter());
-            let (resolve, errors) = parse_attribute_args_just_once(extracted.iter(), "from");
-            self.1.extend(errors.into_iter());
-            if pos.is_some() || resolve.is_some() {
-                self.0
-                    .push(Fixture::new(name, resolve, pos.unwrap_or_default()))
-            }
+        let (pos, errors) = parse_attribute_args_just_once(extracted.iter(), "with");
+        self.1.extend(errors);
+        let (resolve, errors) = parse_attribute_args_just_once(extracted.iter(), "from");
+        self.1.extend(errors);
+        if pos.is_some() || resolve.is_some() {
+            self.0
+                .push(Fixture::new(name, resolve, pos.unwrap_or_default()))
         }
     }
 }
@@ -294,20 +304,6 @@ impl FixtureModifiers {
         ))
     }
 
-    pub(crate) fn set_once(&mut self, once: syn::Ident) {
-        self.inner.attributes.push(Attribute::Attr(once))
-    }
-
-    pub(crate) fn get_once(&self) -> Option<&Ident> {
-        self.iter()
-            .find(|&a| a == &Attribute::Attr(format_ident!("once")))
-            .and_then(|a| a.maybe_ident())
-    }
-
-    pub(crate) fn is_once(&self) -> bool {
-        self.get_once().is_some()
-    }
-
     fn extract_type(&self, attr_name: &str) -> Option<syn::ReturnType> {
         self.iter()
             .filter_map(|m| match m {
@@ -352,6 +348,7 @@ mod should {
                     ],
                 }
                 .into(),
+                arguments: Default::default(),
             };
 
             assert_eq!(expected, data);
@@ -451,6 +448,8 @@ mod extend {
                     fn test_fn(
                         #[from(long_fixture_name)] 
                         #[with(42, "other")] short: u32, 
+                        #[from(sub_module::fix)]
+                        f: u32,
                         #[from(simple)]
                         s: &str,
                         no_change: i32) {
@@ -463,6 +462,7 @@ mod extend {
                     fixture("short", &["42", r#""other""#])
                         .with_resolve("long_fixture_name")
                         .into(),
+                    fixture("f", &[]).with_resolve("sub_module::fix").into(),
                     fixture("s", &[]).with_resolve("simple").into(),
                 ]
                 .into(),
@@ -572,7 +572,7 @@ mod extend {
 
             info.extend_with_function_attrs(&mut item_fn).unwrap();
 
-            assert!(info.attributes.is_once());
+            assert!(info.arguments.is_once());
         }
 
         #[test]
@@ -586,7 +586,21 @@ mod extend {
 
             info.extend_with_function_attrs(&mut item_fn).unwrap();
 
-            assert!(!info.attributes.is_once());
+            assert!(!info.arguments.is_once());
+        }
+
+        #[rstest]
+        fn extract_future() {
+            let mut item_fn = "fn f(#[future] a: u32, b: u32) {}".ast();
+            let expected = "fn f(a: u32, b: u32) {}".ast();
+
+            let mut info = FixtureInfo::default();
+
+            info.extend_with_function_attrs(&mut item_fn).unwrap();
+
+            assert_eq!(item_fn, expected);
+            assert!(info.arguments.is_future(&ident("a")));
+            assert!(!info.arguments.is_future(&ident("b")));
         }
 
         mod raise_error {
@@ -652,6 +666,38 @@ mod extend {
                 assert_eq!(3, errors.len());
             }
 
+            #[test]
+            fn future_is_used_more_than_once() {
+                let mut item_fn: ItemFn = r#"
+                    fn my_fix(#[future] #[future] fixture1: u32) {}
+                "#
+                .ast();
+
+                let errors = FixtureInfo::default()
+                    .extend_with_function_attrs(&mut item_fn)
+                    .err()
+                    .unwrap_or_default();
+
+                assert_eq!(1, errors.len());
+                assert_in!(errors[0].to_string(), "more than once");
+            }
+
+            #[test]
+            fn default_used_more_than_once() {
+                let mut item_fn: ItemFn = r#"
+                    fn my_fix(#[default(2)] #[default(3)] f1: u32) {}
+                "#
+                .ast();
+
+                let errors = FixtureInfo::default()
+                    .extend_with_function_attrs(&mut item_fn)
+                    .err()
+                    .unwrap_or_default();
+
+                assert_eq!(1, errors.len());
+                assert_in!(errors[0].to_string(), "more than once");
+            }
+
             #[test]
             fn if_once_is_defined_more_than_once() {
                 let mut item_fn: ItemFn = r#"
@@ -686,7 +732,7 @@ mod extend {
 
                 assert_in!(
                     format!("{:?}", error).to_lowercase(),
-                    "cannot use default more than once"
+                    "cannot use #[default] more than once"
                 );
             }
 
