@@ -3,31 +3,35 @@ use syn::{
     parse::{Parse, ParseStream},
     parse_quote,
     punctuated::Punctuated,
-    token::{self, Paren},
+    token::{self, Async, Paren},
     visit_mut::VisitMut,
     FnArg, Ident, ItemFn, Token,
 };
 
 use crate::{
     error::ErrorsVec,
+    parse::just_once::{AttrBuilder, JustOnceFnAttributeExtractor, Validator},
     refident::{MaybeIdent, RefIdent},
     utils::{attr_is, attr_starts_with},
 };
-use fixture::{
-    ArgumentValue, DefaultsFunctionExtractor, FixtureModifiers, FixturesFunctionExtractor,
-};
+use fixture::{ArgumentValue, FixtureModifiers, FixturesFunctionExtractor};
 use quote::ToTokens;
 use testcase::TestCase;
 
-use self::{expressions::Expressions, vlist::ValueList};
+use self::{
+    expressions::Expressions, just_once::JustOnceFnArgAttributeExtractor, vlist::ValueList,
+};
 
 // To use the macros this should be the first one module
 #[macro_use]
 pub(crate) mod macros;
 
+pub(crate) mod arguments;
+pub(crate) mod by_ref;
 pub(crate) mod expressions;
 pub(crate) mod fixture;
 pub(crate) mod future;
+pub(crate) mod just_once;
 pub(crate) mod rstest;
 pub(crate) mod testcase;
 pub(crate) mod vlist;
@@ -133,12 +137,12 @@ impl Parse for Positional {
 #[derive(PartialEq, Debug, Clone)]
 pub(crate) struct Fixture {
     pub(crate) name: Ident,
-    pub(crate) resolve: Option<Ident>,
+    pub(crate) resolve: Option<syn::Path>,
     pub(crate) positional: Positional,
 }
 
 impl Fixture {
-    pub(crate) fn new(name: Ident, resolve: Option<Ident>, positional: Positional) -> Self {
+    pub(crate) fn new(name: Ident, resolve: Option<syn::Path>, positional: Positional) -> Self {
         Self {
             name,
             resolve,
@@ -149,7 +153,7 @@ impl Fixture {
 
 impl Parse for Fixture {
     fn parse(input: ParseStream) -> syn::Result<Self> {
-        let resolve = input.parse()?;
+        let resolve: syn::Path = input.parse()?;
         if input.peek(Paren) || input.peek(Token![as]) {
             let positional = if input.peek(Paren) {
                 let content;
@@ -163,7 +167,13 @@ impl Parse for Fixture {
                 let _: Token![as] = input.parse()?;
                 Ok(Self::new(input.parse()?, Some(resolve), positional))
             } else {
-                Ok(Self::new(resolve, None, positional))
+                let name = resolve.get_ident().ok_or_else(|| {
+                    syn::Error::new_spanned(
+                        resolve.to_token_stream(),
+                        "Should be an ident".to_string(),
+                    )
+                })?;
+                Ok(Self::new(name.clone(), None, positional))
             }
         } else {
             Err(syn::Error::new(
@@ -196,24 +206,42 @@ pub(crate) fn extract_fixtures(item_fn: &mut ItemFn) -> Result<Vec<Fixture>, Err
         Err(fixtures_extractor.1.into())
     }
 }
-
 pub(crate) fn extract_defaults(item_fn: &mut ItemFn) -> Result<Vec<ArgumentValue>, ErrorsVec> {
-    let mut defaults_extractor = DefaultsFunctionExtractor::default();
-    defaults_extractor.visit_item_fn_mut(item_fn);
+    struct DefaultBuilder;
+    impl AttrBuilder<Ident> for DefaultBuilder {
+        type Out = ArgumentValue;
 
-    if defaults_extractor.1.is_empty() {
-        Ok(defaults_extractor.0)
-    } else {
-        Err(defaults_extractor.1.into())
+        fn build(attr: syn::Attribute, name: &Ident) -> syn::Result<Self::Out> {
+            attr.parse_args::<syn::Expr>()
+                .map(|e| ArgumentValue::new(name.clone(), e))
+        }
     }
+    impl Validator<syn::FnArg> for DefaultBuilder {}
+
+    let mut extractor = JustOnceFnArgAttributeExtractor::<DefaultBuilder>::new("default");
+    extractor.visit_item_fn_mut(item_fn);
+
+    extractor.take()
 }
 
 pub(crate) fn extract_default_return_type(
     item_fn: &mut ItemFn,
 ) -> Result<Option<syn::Type>, ErrorsVec> {
-    let mut default_type_extractor = DefaultTypeFunctionExtractor::default();
-    default_type_extractor.visit_item_fn_mut(item_fn);
-    default_type_extractor.take()
+    struct DefaultTypeBuilder;
+    impl AttrBuilder<ItemFn> for DefaultTypeBuilder {
+        type Out = syn::Type;
+
+        fn build(attr: syn::Attribute, _extra: &ItemFn) -> syn::Result<Self::Out> {
+            attr.parse_args::<syn::Type>()
+        }
+    }
+    impl Validator<syn::ItemFn> for DefaultTypeBuilder {}
+
+    let mut extractor =
+        JustOnceFnAttributeExtractor::<DefaultTypeBuilder>::new(FixtureModifiers::DEFAULT_RET_ATTR);
+
+    extractor.visit_item_fn_mut(item_fn);
+    extractor.take()
 }
 
 pub(crate) fn extract_partials_return_type(
@@ -224,13 +252,14 @@ pub(crate) fn extract_partials_return_type(
     partials_type_extractor.take()
 }
 
-pub(crate) fn extract_once(item_fn: &mut ItemFn) -> Result<Option<Ident>, ErrorsVec> {
-    let mut extractor = IsOnceAttributeFunctionExtractor::default();
+pub(crate) fn extract_once(item_fn: &mut ItemFn) -> Result<Option<syn::Attribute>, ErrorsVec> {
+    let mut extractor = JustOnceFnAttributeExtractor::from("once");
+
     extractor.visit_item_fn_mut(item_fn);
     extractor.take()
 }
 
-fn extract_argument_attrs<'a, B: 'a + std::fmt::Debug>(
+pub(crate) fn extract_argument_attrs<'a, B: 'a + std::fmt::Debug>(
     node: &mut FnArg,
     is_valid_attr: fn(&syn::Attribute) -> bool,
     build: fn(syn::Attribute, &Ident) -> syn::Result<B>,
@@ -255,51 +284,6 @@ fn extract_argument_attrs<'a, B: 'a + std::fmt::Debug>(
     }
 }
 
-/// Simple struct used to visit function attributes and extract default return
-/// type
-struct DefaultTypeFunctionExtractor(Result<Option<syn::Type>, ErrorsVec>);
-
-impl DefaultTypeFunctionExtractor {
-    fn take(self) -> Result<Option<syn::Type>, ErrorsVec> {
-        self.0
-    }
-}
-
-impl Default for DefaultTypeFunctionExtractor {
-    fn default() -> Self {
-        Self(Ok(None))
-    }
-}
-
-impl VisitMut for DefaultTypeFunctionExtractor {
-    fn visit_item_fn_mut(&mut self, node: &mut ItemFn) {
-        let attrs = std::mem::take(&mut node.attrs);
-        let (defaults, remain): (Vec<_>, Vec<_>) = attrs
-            .into_iter()
-            .partition(|attr| attr_is(attr, FixtureModifiers::DEFAULT_RET_ATTR));
-
-        node.attrs = remain;
-        let mut defaults = defaults.into_iter();
-        let mut data = None;
-        let mut errors = ErrorsVec::default();
-        match defaults.next().map(|def| def.parse_args::<syn::Type>()) {
-            Some(Ok(t)) => data = Some(t),
-            Some(Err(e)) => errors.push(e),
-            None => {}
-        };
-        errors.extend(
-            defaults.map(|a| syn::Error::new_spanned(a, "You cannot use default more than once")),
-        );
-        self.0 = if errors.len() > 0 {
-            Err(errors)
-        } else {
-            Ok(data)
-        };
-
-        syn::visit_mut::visit_item_fn_mut(self, node);
-    }
-}
-
 /// Simple struct used to visit function attributes and extract default return
 /// type
 struct PartialsTypeFunctionExtractor(Result<Vec<(usize, syn::Type)>, ErrorsVec>);
@@ -322,7 +306,7 @@ impl VisitMut for PartialsTypeFunctionExtractor {
         let (partials, remain): (Vec<_>, Vec<_>) =
             attrs
                 .into_iter()
-                .partition(|attr| match attr.path.get_ident() {
+                .partition(|attr| match attr.path().get_ident() {
                     Some(name) => name
                         .to_string()
                         .starts_with(FixtureModifiers::PARTIAL_RET_ATTR),
@@ -335,7 +319,7 @@ impl VisitMut for PartialsTypeFunctionExtractor {
         for attr in partials {
             match attr.parse_args::<syn::Type>() {
                 Ok(t) => {
-                    match attr.path.get_ident().unwrap().to_string()
+                    match attr.path().get_ident().unwrap().to_string()
                         [FixtureModifiers::PARTIAL_RET_ATTR.len()..]
                         .parse()
                     {
@@ -354,48 +338,8 @@ impl VisitMut for PartialsTypeFunctionExtractor {
         } else {
             Ok(data)
         };
-
-        syn::visit_mut::visit_item_fn_mut(self, node);
     }
 }
-
-/// Simple struct used to visit function attributes and extract once
-/// type
-struct IsOnceAttributeFunctionExtractor(Result<Option<Ident>, ErrorsVec>);
-
-impl IsOnceAttributeFunctionExtractor {
-    fn take(self) -> Result<Option<Ident>, ErrorsVec> {
-        self.0
-    }
-}
-
-impl Default for IsOnceAttributeFunctionExtractor {
-    fn default() -> Self {
-        Self(Ok(None))
-    }
-}
-
-impl VisitMut for IsOnceAttributeFunctionExtractor {
-    fn visit_item_fn_mut(&mut self, node: &mut ItemFn) {
-        let attrs = std::mem::take(&mut node.attrs);
-        let (onces, remain): (Vec<_>, Vec<_>) =
-            attrs.into_iter().partition(|attr| attr_is(attr, "once"));
-
-        node.attrs = remain;
-        self.0 = match onces.len() {
-            1 => Ok(onces[0].path.get_ident().cloned()),
-            0 => Ok(None),
-            _ => Err(onces
-                .into_iter()
-                .skip(1)
-                .map(|attr| syn::Error::new_spanned(attr, "You cannot use #[once] more than once"))
-                .collect::<Vec<_>>()
-                .into()),
-        };
-        syn::visit_mut::visit_item_fn_mut(self, node);
-    }
-}
-
 /// Simple struct used to visit function attributes and extract case arguments and
 /// eventualy parsing errors
 #[derive(Default)]
@@ -415,14 +359,10 @@ impl VisitMut for CaseArgsFunctionExtractor {
 }
 
 pub(crate) fn extract_case_args(item_fn: &mut ItemFn) -> Result<Vec<Ident>, ErrorsVec> {
-    let mut case_args_extractor = CaseArgsFunctionExtractor::default();
-    case_args_extractor.visit_item_fn_mut(item_fn);
+    let mut extractor = JustOnceFnArgAttributeExtractor::from("case");
+    extractor.visit_item_fn_mut(item_fn);
 
-    if case_args_extractor.1.is_empty() {
-        Ok(case_args_extractor.0)
-    } else {
-        Err(case_args_extractor.1.into())
-    }
+    extractor.take()
 }
 
 /// Simple struct used to visit function attributes and extract cases and
@@ -439,7 +379,8 @@ impl VisitMut for CasesFunctionExtractor {
             if attr_starts_with(&attr, &case) {
                 match attr.parse_args::<Expressions>() {
                     Ok(expressions) => {
-                        let description = attr.path.segments.into_iter().nth(1).map(|p| p.ident);
+                        let description =
+                            attr.path().segments.iter().nth(1).map(|p| p.ident.clone());
                         self.0.push(TestCase {
                             args: expressions.into(),
                             attrs: std::mem::take(&mut attrs_buffer),
@@ -453,7 +394,6 @@ impl VisitMut for CasesFunctionExtractor {
             }
         }
         node.attrs = std::mem::take(&mut attrs_buffer);
-        syn::visit_mut::visit_item_fn_mut(self, node);
     }
 }
 
@@ -468,42 +408,24 @@ pub(crate) fn extract_cases(item_fn: &mut ItemFn) -> Result<Vec<TestCase>, Error
     }
 }
 
-/// Simple struct used to visit function attributes and extract value list and
-/// eventualy parsing errors
-#[derive(Default)]
-struct ValueListFunctionExtractor(Vec<ValueList>, Vec<syn::Error>);
-
-impl VisitMut for ValueListFunctionExtractor {
-    fn visit_fn_arg_mut(&mut self, node: &mut FnArg) {
-        for r in extract_argument_attrs(
-            node,
-            |a| attr_is(a, "values"),
-            |a, name| {
-                a.parse_args::<Expressions>().map(|v| ValueList {
-                    arg: name.clone(),
-                    values: v.take(),
-                })
-            },
-        ) {
-            match r {
-                Ok(vlist) => self.0.push(vlist),
-                Err(err) => self.1.push(err),
-            }
-        }
-
-        syn::visit_mut::visit_fn_arg_mut(self, node);
-    }
-}
-
 pub(crate) fn extract_value_list(item_fn: &mut ItemFn) -> Result<Vec<ValueList>, ErrorsVec> {
-    let mut vlist_extractor = ValueListFunctionExtractor::default();
-    vlist_extractor.visit_item_fn_mut(item_fn);
+    struct ValueListBuilder;
+    impl AttrBuilder<Ident> for ValueListBuilder {
+        type Out = ValueList;
 
-    if vlist_extractor.1.is_empty() {
-        Ok(vlist_extractor.0)
-    } else {
-        Err(vlist_extractor.1.into())
+        fn build(attr: syn::Attribute, extra: &Ident) -> syn::Result<Self::Out> {
+            attr.parse_args::<Expressions>().map(|v| ValueList {
+                arg: extra.clone(),
+                values: v.take().into_iter().map(|e| e.into()).collect(),
+            })
+        }
     }
+    impl Validator<FnArg> for ValueListBuilder {}
+
+    let mut extractor = JustOnceFnArgAttributeExtractor::<ValueListBuilder>::new("values");
+
+    extractor.visit_item_fn_mut(item_fn);
+    extractor.take()
 }
 
 /// Simple struct used to visit function args attributes to extract the
@@ -573,23 +495,49 @@ impl CheckTimeoutAttributesFunction {
     pub(crate) fn take(self) -> Result<(), ErrorsVec> {
         self.0
     }
+
+    fn check_if_can_implement_timeous(
+        &self,
+        timeouts: &[&syn::Attribute],
+        asyncness: Option<&Async>,
+    ) -> Option<syn::Error> {
+        if cfg!(feature = "async-timeout") || timeouts.is_empty() {
+            None
+        } else {
+            asyncness.map(|a| {
+                syn::Error::new(
+                    a.span,
+                    "Enable async-timeout feature to use timeout in async tests",
+                )
+            })
+        }
+    }
 }
 
 impl Default for CheckTimeoutAttributesFunction {
     fn default() -> Self {
-        Self(Ok(Default::default()))
+        Self(Ok(()))
     }
 }
 
 impl VisitMut for CheckTimeoutAttributesFunction {
     fn visit_item_fn_mut(&mut self, node: &mut ItemFn) {
-        let errors = node
+        let timeouts = node
             .attrs
             .iter()
             .filter(|&a| attr_is(a, "timeout"))
-            .map(|attr| attr.parse_args::<syn::Expr>())
+            .collect::<Vec<_>>();
+        let mut errors = timeouts
+            .iter()
+            .map(|&attr| attr.parse_args::<syn::Expr>())
             .filter_map(Result::err)
             .collect::<Vec<_>>();
+
+        if let Some(e) =
+            self.check_if_can_implement_timeous(timeouts.as_slice(), node.sig.asyncness.as_ref())
+        {
+            errors.push(e);
+        }
         if !errors.is_empty() {
             *self = Self(Err(errors.into()));
         }
