@@ -3,11 +3,17 @@ use syn::{
     Ident, ItemFn, Token,
 };
 
-use super::testcase::TestCase;
+use self::files::{extract_files, ValueListFromFiles};
+
 use super::{
+    arguments::ArgumentsInfo,
+    by_ref::extract_by_ref,
     check_timeout_attrs, extract_case_args, extract_cases, extract_excluded_trace,
-    extract_fixtures, extract_value_list, parse_vector_trailing_till_double_comma, Attribute,
-    Attributes, ExtendWithFunctionAttrs, Fixture,
+    extract_fixtures, extract_value_list,
+    future::{extract_futures, extract_global_awt},
+    parse_vector_trailing_till_double_comma,
+    testcase::TestCase,
+    Attribute, Attributes, ExtendWithFunctionAttrs, Fixture,
 };
 use crate::parse::vlist::ValueList;
 use crate::{
@@ -17,10 +23,13 @@ use crate::{
 use proc_macro2::{Span, TokenStream};
 use quote::{format_ident, ToTokens};
 
+pub(crate) mod files;
+
 #[derive(PartialEq, Debug, Default)]
 pub(crate) struct RsTestInfo {
     pub(crate) data: RsTestData,
     pub(crate) attributes: RsTestAttributes,
+    pub(crate) arguments: ArgumentsInfo,
 }
 
 impl Parse for RsTestInfo {
@@ -34,6 +43,7 @@ impl Parse for RsTestInfo {
                     .parse::<Token![::]>()
                     .or_else(|_| Ok(Default::default()))
                     .and_then(|_| input.parse())?,
+                arguments: Default::default(),
             }
         })
     }
@@ -41,12 +51,18 @@ impl Parse for RsTestInfo {
 
 impl ExtendWithFunctionAttrs for RsTestInfo {
     fn extend_with_function_attrs(&mut self, item_fn: &mut ItemFn) -> Result<(), ErrorsVec> {
-        let (_, (excluded, _)) = merge_errors!(
+        let composed_tuple!(_inner, excluded, _timeout, futures, global_awt, by_refs) = merge_errors!(
             self.data.extend_with_function_attrs(item_fn),
             extract_excluded_trace(item_fn),
-            check_timeout_attrs(item_fn)
+            check_timeout_attrs(item_fn),
+            extract_futures(item_fn),
+            extract_global_awt(item_fn),
+            extract_by_ref(item_fn)
         )?;
         self.attributes.add_notraces(excluded);
+        self.arguments.set_global_await(global_awt);
+        self.arguments.set_futures(futures.into_iter());
+        self.arguments.set_by_refs(by_refs.into_iter());
         Ok(())
     }
 }
@@ -118,17 +134,24 @@ impl Parse for RsTestData {
 
 impl ExtendWithFunctionAttrs for RsTestData {
     fn extend_with_function_attrs(&mut self, item_fn: &mut ItemFn) -> Result<(), ErrorsVec> {
-        let composed_tuple!(fixtures, case_args, cases, value_list) = merge_errors!(
+        let composed_tuple!(fixtures, case_args, cases, value_list, files) = merge_errors!(
             extract_fixtures(item_fn),
             extract_case_args(item_fn),
             extract_cases(item_fn),
-            extract_value_list(item_fn)
+            extract_value_list(item_fn),
+            extract_files(item_fn)
         )?;
 
         self.items.extend(fixtures.into_iter().map(|f| f.into()));
         self.items.extend(case_args.into_iter().map(|f| f.into()));
         self.items.extend(cases.into_iter().map(|f| f.into()));
         self.items.extend(value_list.into_iter().map(|f| f.into()));
+        self.items.extend(
+            ValueListFromFiles::default()
+                .to_value_list(files)?
+                .into_iter()
+                .map(|f| f.into()),
+        );
         Ok(())
     }
 }
@@ -261,6 +284,7 @@ impl Parse for RsTestAttributes {
 mod test {
     use super::*;
     use crate::test::{assert_eq, *};
+    use rstest_test::assert_in;
 
     mod parse_rstest_data {
         use super::assert_eq;
@@ -301,13 +325,45 @@ mod test {
         assert_eq!(2, errors.len());
     }
 
+    #[cfg(feature = "async-timeout")]
+    #[test]
+    fn should_parse_async_timeout() {
+        let mut item_fn = r#"
+            #[timeout(Duration::from_millis(20))]
+            async fn test_fn(#[case] arg: u32) {
+            }
+        "#
+        .ast();
+
+        let mut info = RsTestInfo::default();
+
+        info.extend_with_function_attrs(&mut item_fn).unwrap();
+    }
+
+    #[cfg(not(feature = "async-timeout"))]
+    #[test]
+    fn should_return_error_for_async_timeout() {
+        let mut item_fn = r#"
+            #[timeout(Duration::from_millis(20))]
+            async fn test_fn(#[case] arg: u32) {
+            }
+        "#
+        .ast();
+
+        let mut info = RsTestInfo::default();
+
+        let errors = info.extend_with_function_attrs(&mut item_fn).unwrap_err();
+
+        assert_eq!(1, errors.len());
+        assert!(format!("{:?}", errors).contains("async-timeout feature"))
+    }
+
     fn parse_rstest<S: AsRef<str>>(rstest_data: S) -> RsTestInfo {
         parse_meta(rstest_data)
     }
 
     mod no_cases {
         use super::{assert_eq, *};
-        use crate::parse::{Attribute, Attributes};
 
         #[test]
         fn happy_path() {
@@ -329,6 +385,7 @@ mod test {
                     ],
                 }
                 .into(),
+                ..Default::default()
             };
 
             assert_eq!(expected, data);
@@ -340,7 +397,7 @@ mod test {
             #[test]
             fn rename() {
                 let data = parse_rstest(
-                    r#"long_fixture_name(42, "other") as short, simple as s, no_change()"#,
+                    r#"long_fixture_name(42, "other") as short, sub_module::fix as f, simple as s, no_change()"#,
                 );
 
                 let expected = RsTestInfo {
@@ -348,6 +405,7 @@ mod test {
                         fixture("short", &["42", r#""other""#])
                             .with_resolve("long_fixture_name")
                             .into(),
+                        fixture("f", &[]).with_resolve("sub_module::fix").into(),
                         fixture("s", &[]).with_resolve("simple").into(),
                         fixture("no_change", &[]).into(),
                     ]
@@ -366,6 +424,8 @@ mod test {
                         #[with(42, "other")] short: u32, 
                         #[from(simple)]
                         s: &str,
+                        #[from(sub_module::fix)]
+                        f: u32,
                         no_change: i32) {
                     }
                     "#
@@ -377,6 +437,7 @@ mod test {
                             .with_resolve("long_fixture_name")
                             .into(),
                         fixture("s", &[]).with_resolve("simple").into(),
+                        fixture("f", &[]).with_resolve("sub_module::fix").into(),
                     ]
                     .into(),
                     ..Default::default()
@@ -472,11 +533,24 @@ mod test {
                 .unwrap();
             assert_eq!(attrs("#[something_else]"), b_args);
         }
+
+        #[rstest]
+        fn extract_future() {
+            let mut item_fn = "fn f(#[future] a: u32, b: u32) {}".ast();
+            let expected = "fn f(a: u32, b: u32) {}".ast();
+
+            let mut info = RsTestInfo::default();
+
+            info.extend_with_function_attrs(&mut item_fn).unwrap();
+
+            assert_eq!(item_fn, expected);
+            assert!(info.arguments.is_future(&ident("a")));
+            assert!(!info.arguments.is_future(&ident("b")));
+        }
     }
 
     mod parametrize_cases {
         use super::{assert_eq, *};
-        use std::iter::FromIterator;
 
         #[test]
         fn one_simple_case_one_arg() {
@@ -732,10 +806,26 @@ mod test {
             assert_eq!(1, cases.len());
             assert_eq!(to_args!(["42"]), cases[0].args());
         }
+
+        #[test]
+        fn should_reject_case_args_marked_more_than_once() {
+            let mut item_fn = r#"
+                    #[case(42)]
+                    fn test_fn(#[case] #[case] arg: u32) {
+                    }
+                "#
+            .ast();
+
+            let mut info = RsTestInfo::default();
+
+            let errors = info.extend_with_function_attrs(&mut item_fn).unwrap_err();
+
+            assert_eq!(1, errors.len());
+            assert_in!(errors[0].to_string(), "more than once");
+        }
     }
 
     mod matrix_cases {
-        use crate::parse::Attribute;
 
         use super::{assert_eq, *};
 
@@ -832,6 +922,22 @@ mod test {
                 );
             }
         }
+
+        #[test]
+        fn should_reject_values_attribute_marked_more_than_once() {
+            let mut item_fn = r#"
+                fn test_fn(#[values(1, 2, 1+2)] #[values(1, 2, 1+2)] arg1: u32, ) {
+                }
+                "#
+            .ast();
+
+            let mut info = RsTestInfo::default();
+
+            let errors = info.extend_with_function_attrs(&mut item_fn).unwrap_err();
+
+            assert_eq!(1, errors.len());
+            assert_in!(errors[0].to_string(), "more than once");
+        }
     }
 
     mod integrated {
