@@ -4,7 +4,7 @@ use syn::{
     parse::{Parse, ParseStream, Result},
     parse2, parse_str,
     visit::Visit,
-    ItemFn, ItemMod,
+    ItemFn, ItemMod, LocalInit,
 };
 
 use super::*;
@@ -36,7 +36,10 @@ fn trace_argument_code_string(arg_name: &str) -> String {
 mod single_test_should {
     use rstest_test::{assert_in, assert_not_in};
 
-    use crate::test::{assert_eq, *};
+    use crate::{
+        parse::arguments::{ArgumentsInfo, FutureArg},
+        test::{assert_eq, *},
+    };
 
     use super::*;
 
@@ -49,6 +52,14 @@ mod single_test_should {
         assert_eq!(result.sig.output, input_fn.sig.output);
     }
 
+    fn extract_inner_test_function(outer: &ItemFn) -> ItemFn {
+        let first_stmt = outer.block.stmts.get(0).unwrap();
+
+        parse_quote! {
+            #first_stmt
+        }
+    }
+
     #[test]
     fn include_given_function() {
         let input_fn: ItemFn = r#"
@@ -61,13 +72,26 @@ mod single_test_should {
                 "#.ast();
 
         let result: ItemFn = single(input_fn.clone(), Default::default()).ast();
-        let first_stmt = result.block.stmts.get(0).unwrap();
 
-        let inner_fn: ItemFn = parse_quote! {
-            #first_stmt
-        };
+        let inner_fn = extract_inner_test_function(&result);
+        let inner_fn_impl: Stmt = inner_fn.block.stmts.last().cloned().unwrap();
 
-        assert_eq!(inner_fn, input_fn);
+        assert_eq!(inner_fn.sig, input_fn.sig);
+        assert_eq!(inner_fn_impl.display_code(), input_fn.block.display_code());
+    }
+
+    #[test]
+    fn not_remove_lifetimes() {
+        let input_fn: ItemFn = r#"
+                pub fn test<'a, 'b, 'c: 'a + 'b>(a: A<'a>, b: A<'b>, c: A<'c>) -> A<'c>
+                {
+                }
+                "#
+        .ast();
+
+        let result: ItemFn = single(input_fn.clone(), Default::default()).ast();
+
+        assert_eq!(3, result.sig.generics.lifetimes().count());
     }
 
     #[rstest]
@@ -120,6 +144,76 @@ mod single_test_should {
         assert_eq!(result.attrs, attributes);
     }
 
+    #[test]
+    fn use_global_await() {
+        let input_fn: ItemFn = r#"fn test(a: i32, b:i32, c:i32) {} "#.ast();
+        let mut info: RsTestInfo = Default::default();
+        info.arguments.set_global_await(true);
+        info.arguments.add_future(ident("a"));
+        info.arguments.add_future(ident("b"));
+
+        let item_fn: ItemFn = single(input_fn.clone(), info).ast();
+
+        assert_in!(
+            item_fn.block.display_code(),
+            await_argument_code_string("a")
+        );
+        assert_in!(
+            item_fn.block.display_code(),
+            await_argument_code_string("b")
+        );
+        assert_not_in!(
+            item_fn.block.display_code(),
+            await_argument_code_string("c")
+        );
+    }
+
+    #[test]
+    fn use_selective_await() {
+        let input_fn: ItemFn = r#"fn test(a: i32, b:i32, c:i32) {} "#.ast();
+        let mut info: RsTestInfo = Default::default();
+        info.arguments.set_future(ident("a"), FutureArg::Define);
+        info.arguments.set_future(ident("b"), FutureArg::Await);
+
+        let item_fn: ItemFn = single(input_fn.clone(), info).ast();
+
+        assert_not_in!(
+            item_fn.block.display_code(),
+            await_argument_code_string("a",)
+        );
+        assert_in!(
+            item_fn.block.display_code(),
+            await_argument_code_string("b")
+        );
+        assert_not_in!(
+            item_fn.block.display_code(),
+            await_argument_code_string("c")
+        );
+    }
+
+    #[test]
+    fn use_ref_if_any() {
+        let input_fn: ItemFn = r#"fn test(a: i32, b:i32, c:i32) {} "#.ast();
+        let mut info: RsTestInfo = Default::default();
+        info.arguments.set_by_ref(ident("a"));
+        info.arguments.set_by_ref(ident("c"));
+
+        let item_fn: ItemFn = single(input_fn.clone(), info).ast();
+
+        assert_in!(
+            item_fn.block.stmts.last().display_code(),
+            ref_argument_code_string("a")
+        );
+        assert_not_in!(
+            item_fn.block.stmts.last().display_code(),
+            ref_argument_code_string("b")
+        );
+        assert_in!(
+            item_fn.block.stmts.last().display_code(),
+            ref_argument_code_string("c")
+        );
+    }
+
     #[test]
     fn trace_arguments_values() {
         let input_fn: ItemFn = r#"#[trace]fn test(s: String, a:i32) {} "#.ast();
@@ -210,6 +304,39 @@ mod single_test_should {
 
         assert_eq!(use_await, last_stmt.is_await());
     }
+    #[test]
+    fn add_future_boilerplate_if_requested() {
+        let item_fn: ItemFn = r#"
+                    async fn test(async_ref_u32: &u32, async_u32: u32,simple: u32)
+                    { }
+                     "#
+        .ast();
+
+        let mut arguments = ArgumentsInfo::default();
+        arguments.add_future(ident("async_ref_u32"));
+        arguments.add_future(ident("async_u32"));
+
+        let info = RsTestInfo {
+            arguments,
+            ..Default::default()
+        };
+
+        let result: ItemFn = single(item_fn.clone(), info).ast();
+        let inner_fn = extract_inner_test_function(&result);
+
+        let expected = parse_str::<syn::ItemFn>(
+            r#"async fn test<'_async_ref_u32>(
+                        async_ref_u32: impl std::future::Future<Output = &'_async_ref_u32 u32>, 
+                        async_u32: impl std::future::Future<Output = u32>, 
+                        simple: u32
+                    )
+                    { }
+                    "#,
+        )
+        .unwrap();
+
+        assert_eq!(inner_fn.sig, expected.sig);
+    }
 }
 
 struct TestsGroup {
@@ -227,13 +354,14 @@ impl Parse for TestsGroup {
 }
 
 trait QueryAttrs {
+    #[allow(dead_code)]
     fn has_attr(&self, attr: &syn::Path) -> bool;
     fn has_attr_that_ends_with(&self, attr: &syn::PathSegment) -> bool;
 }
 
 impl QueryAttrs for ItemFn {
     fn has_attr(&self, attr: &syn::Path) -> bool {
-        self.attrs.iter().find(|a| &a.path == attr).is_some()
+        self.attrs.iter().find(|a| a.path() == attr).is_some()
     }
 
     fn has_attr_that_ends_with(&self, name: &syn::PathSegment) -> bool {
@@ -365,7 +493,7 @@ impl<'ast> Visit<'ast> for Assignments {
         match &assign {
             syn::Local {
                 pat: syn::Pat::Ident(pat),
-                init: Some((_, expr)),
+                init: Some(LocalInit { expr, .. }),
                 ..
             } => {
                 self.0.insert(pat.ident.to_string(), expr.as_ref().clone());
@@ -390,13 +518,12 @@ impl From<TokenStream> for TestsGroup {
 }
 
 mod cases_should {
-    use std::iter::FromIterator;
 
     use rstest_test::{assert_in, assert_not_in};
 
     use crate::parse::{
-        rstest::{RsTestData, RsTestInfo, RsTestItem},
-        testcase::TestCase,
+        arguments::{ArgumentsInfo, FutureArg},
+        rstest::RsTestItem,
     };
 
     use super::{assert_eq, *};
@@ -484,9 +611,11 @@ mod cases_should {
         let tokens = parametrize(item_fn.clone(), info);
 
         let mut output = TestsGroup::from(tokens);
+        let test_impl: Stmt = output.requested_test.block.stmts.last().cloned().unwrap();
 
         output.requested_test.attrs = vec![];
-        assert_eq!(output.requested_test, item_fn);
+        assert_eq!(output.requested_test.sig, item_fn.sig);
+        assert_eq!(test_impl.display_code(), item_fn.block.display_code());
     }
 
     #[test]
@@ -761,6 +890,36 @@ mod cases_should {
         assert_eq!(&tests[0].attrs[1..], attributes.as_slice());
     }
 
+    #[test]
+    fn add_future_boilerplate_if_requested() {
+        let (item_fn, mut info) = TestCaseBuilder::from(
+            r#"async fn test(async_ref_u32: &u32, async_u32: u32,simple: u32) { }"#,
+        )
+        .take();
+
+        let mut arguments = ArgumentsInfo::default();
+        arguments.add_future(ident("async_ref_u32"));
+        arguments.add_future(ident("async_u32"));
+
+        info.arguments = arguments;
+
+        let tokens = parametrize(item_fn.clone(), info);
+        let test_function = TestsGroup::from(tokens).requested_test;
+
+        let expected = parse_str::<syn::ItemFn>(
+            r#"async fn test<'_async_ref_u32>(
+                        async_ref_u32: impl std::future::Future<Output = &'_async_ref_u32 u32>, 
+                        async_u32: impl std::future::Future<Output = u32>, 
+                        simple: u32
+                    )
+                    { }
+                    "#,
+        )
+        .unwrap();
+
+        assert_eq!(test_function.sig, expected.sig);
+    }
+
     #[rstest]
     #[case::sync(false, false)]
     #[case::async_fn(true, true)]
@@ -856,12 +1015,53 @@ mod cases_should {
             trace_argument_code_string("a_no_trace_me")
         );
     }
+
+    #[test]
+    fn use_global_await() {
+        let (item_fn, mut info) = TestCaseBuilder::from(r#"fn test(a: i32, b:i32, c:i32) {}"#)
+            .push_case(TestCase::from_iter(vec!["1", "2", "3"]))
+            .push_case(TestCase::from_iter(vec!["1", "2", "3"]))
+            .take();
+        info.arguments.set_global_await(true);
+        info.arguments.add_future(ident("a"));
+        info.arguments.add_future(ident("b"));
+
+        let tokens = parametrize(item_fn, info);
+
+        let tests = TestsGroup::from(tokens);
+
+        let code = tests.requested_test.block.display_code();
+
+        assert_in!(code, await_argument_code_string("a"));
+        assert_in!(code, await_argument_code_string("b"));
+        assert_not_in!(code, await_argument_code_string("c"));
+    }
+
+    #[test]
+    fn use_selective_await() {
+        let (item_fn, mut info) = TestCaseBuilder::from(r#"fn test(a: i32, b:i32, c:i32) {}"#)
+            .push_case(TestCase::from_iter(vec!["1", "2", "3"]))
+            .push_case(TestCase::from_iter(vec!["1", "2", "3"]))
+            .take();
+        info.arguments.set_future(ident("a"), FutureArg::Define);
+        info.arguments.set_future(ident("b"), FutureArg::Await);
+
+        let tokens = parametrize(item_fn, info);
+
+        let tests = TestsGroup::from(tokens);
+
+        let code = tests.requested_test.block.display_code();
+
+        assert_not_in!(code, await_argument_code_string("a"));
+        assert_in!(code, await_argument_code_string("b"));
+        assert_not_in!(code, await_argument_code_string("c"));
+    }
 }
 
 mod matrix_cases_should {
     use rstest_test::{assert_in, assert_not_in};
 
-    use crate::parse::vlist::ValueList;
+    use crate::parse::arguments::{ArgumentsInfo, FutureArg};
 
     /// Should test matrix tests render without take in account MatrixInfo to RsTestInfo
     /// transformation
@@ -903,9 +1103,11 @@ mod matrix_cases_should {
         let tokens = matrix(item_fn.clone(), data.into());
 
         let mut output = TestsGroup::from(tokens);
+        let test_impl: Stmt = output.requested_test.block.stmts.last().cloned().unwrap();
 
         output.requested_test.attrs = vec![];
-        assert_eq!(output.requested_test, item_fn);
+        assert_eq!(output.requested_test.sig, item_fn.sig);
+        assert_eq!(test_impl.display_code(), item_fn.block.display_code());
     }
 
     #[test]
@@ -968,7 +1170,8 @@ mod matrix_cases_should {
         assert!(tests.len() > 0);
 
         for t in tests {
-            assert_eq!(item_fn.attrs, &t.attrs[1..]);
+            let end = t.attrs.len() - 1;
+            assert_eq!(item_fn.attrs, &t.attrs[1..end]);
         }
     }
 
@@ -1067,6 +1270,7 @@ mod matrix_cases_should {
         attributes: &str,
     ) {
         let attributes = attrs(attributes);
+        let filter = attrs("#[allow(non_snake_case)]");
         let data = RsTestData {
             items: vec![values_list("v", &["1", "2", "3"]).into()].into(),
         };
@@ -1082,7 +1286,12 @@ mod matrix_cases_should {
         assert!(tests.len() > 0);
 
         for test in tests {
-            assert_eq!(attributes, test.attrs);
+            let filterd: Vec<_> = test
+                .attrs
+                .into_iter()
+                .filter(|a| !filter.contains(a))
+                .collect();
+            assert_eq!(attributes, filterd);
         }
     }
 
@@ -1119,7 +1328,71 @@ mod matrix_cases_should {
 
         for test in tests {
             assert_eq!(test.attrs[0], test_attribute);
-            assert_eq!(&test.attrs[1..], attributes.as_slice());
+            assert_eq!(&test.attrs[1..test.attrs.len() - 1], attributes.as_slice());
+        }
+    }
+
+    #[test]
+    fn add_future_boilerplate_if_requested() {
+        let item_fn = r#"async fn test(async_ref_u32: &u32, async_u32: u32,simple: u32) { }"#.ast();
+
+        let mut arguments = ArgumentsInfo::default();
+        arguments.add_future(ident("async_ref_u32"));
+        arguments.add_future(ident("async_u32"));
+
+        let info = RsTestInfo {
+            arguments,
+            ..Default::default()
+        };
+
+        let tokens = matrix(item_fn, info);
+
+        let test_function = TestsGroup::from(tokens).requested_test;
+
+        let expected = parse_str::<syn::ItemFn>(
+            r#"async fn test<'_async_ref_u32>(
+                        async_ref_u32: impl std::future::Future<Output = &'_async_ref_u32 u32>, 
+                        async_u32: impl std::future::Future<Output = u32>, 
+                        simple: u32
+                    )
+                    { }
+                    "#,
+        )
+        .unwrap();
+
+        assert_eq!(test_function.sig, expected.sig);
+    }
+
+    #[rstest]
+    fn add_allow_non_snake_case(
+        #[values(
+            "",
+            "#[no_one]",
+            "#[should_panic]",
+            "#[should_panic]#[other]",
+            "#[a::b::c]#[should_panic]"
+        )]
+        attributes: &str,
+    ) {
+        let attributes = attrs(attributes);
+        let non_snake_case = &attrs("#[allow(non_snake_case)]")[0];
+        let data = RsTestData {
+            items: vec![values_list("v", &["1", "2", "3"]).into()].into(),
+        };
+
+        let mut item_fn: ItemFn = r#"fn test(v: u32) {{ println!("user code") }}"#.ast();
+        item_fn.attrs = attributes.clone();
+
+        let tokens = matrix(item_fn, data.into());
+
+        let tests = TestsGroup::from(tokens).get_all_tests();
+
+        // Sanity check
+        assert!(tests.len() > 0);
+
+        for test in tests {
+            assert_eq!(test.attrs.last().unwrap(), non_snake_case);
+            assert_eq!(&test.attrs[1..test.attrs.len() - 1], attributes.as_slice());
         }
     }
 
@@ -1185,7 +1458,14 @@ mod matrix_cases_should {
         attributes.add_notraces(vec![ident("b_no_trace_me"), ident("c_no_trace_me")]);
         let item_fn: ItemFn = r#"#[trace] fn test(a_trace_me: u32, b_no_trace_me: u32, c_no_trace_me: u32, d_trace_me: u32) {}"#.ast();
 
-        let tokens = matrix(item_fn, RsTestInfo { data, attributes });
+        let tokens = matrix(
+            item_fn,
+            RsTestInfo {
+                data,
+                attributes,
+                ..Default::default()
+            },
+        );
 
         let tests = TestsGroup::from(tokens).get_all_tests();
 
@@ -1206,6 +1486,68 @@ mod matrix_cases_should {
         }
     }
 
+    #[test]
+    fn use_global_await() {
+        let item_fn: ItemFn = r#"fn test(a: i32, b:i32, c:i32) {}"#.ast();
+        let data = RsTestData {
+            items: vec![
+                values_list("a", &["1"]).into(),
+                values_list("b", &["2"]).into(),
+                values_list("c", &["3"]).into(),
+            ]
+            .into(),
+        };
+        let mut info = RsTestInfo {
+            data,
+            attributes: Default::default(),
+            arguments: Default::default(),
+        };
+        info.arguments.set_global_await(true);
+        info.arguments.add_future(ident("a"));
+        info.arguments.add_future(ident("b"));
+
+        let tokens = matrix(item_fn, info);
+
+        let tests = TestsGroup::from(tokens);
+
+        let code = tests.requested_test.block.display_code();
+
+        assert_in!(code, await_argument_code_string("a"));
+        assert_in!(code, await_argument_code_string("b"));
+        assert_not_in!(code, await_argument_code_string("c"));
+    }
+
+    #[test]
+    fn use_selective_await() {
+        let item_fn: ItemFn = r#"fn test(a: i32, b:i32, c:i32) {}"#.ast();
+        let data = RsTestData {
+            items: vec![
+                values_list("a", &["1"]).into(),
+                values_list("b", &["2"]).into(),
+                values_list("c", &["3"]).into(),
+            ]
+            .into(),
+        };
+        let mut info = RsTestInfo {
+            data,
+            attributes: Default::default(),
+            arguments: Default::default(),
+        };
+
+        info.arguments.set_future(ident("a"), FutureArg::Define);
+        info.arguments.set_future(ident("b"), FutureArg::Await);
+
+        let tokens = matrix(item_fn, info);
+
+        let tests = TestsGroup::from(tokens);
+
+        let code = tests.requested_test.block.display_code();
+
+        assert_not_in!(code, await_argument_code_string("a"));
+        assert_in!(code, await_argument_code_string("b"));
+        assert_not_in!(code, await_argument_code_string("c"));
+    }
+
     mod two_args_should {
         /// Should test matrix tests render without take in account MatrixInfo to RsTestInfo
         /// transformation
@@ -1244,7 +1586,24 @@ mod matrix_cases_should {
                 .map(|i| format!("{}_{}", names[0], i))
                 .collect::<Vec<_>>();
 
-            assert_eq!(expected, modules);
+            assert_eq!(expected.len(), modules.len());
+            for (e, m) in expected.into_iter().zip(modules.into_iter()) {
+                assert_in!(m, e);
+            }
+        }
+
+        #[test]
+        fn annotate_modules_with_allow_non_snake_name() {
+            let (_, item_fn, info) = fixture();
+            let non_snake_case = &attrs("#[allow(non_snake_case)]")[0];
+
+            let tokens = matrix(item_fn, info);
+
+            let modules = TestsGroup::from(tokens).module.get_modules();
+
+            for module in modules {
+                assert!(module.attrs.contains(&non_snake_case));
+            }
         }
 
         #[test]
@@ -1289,7 +1648,10 @@ mod matrix_cases_should {
                 .map(|i| format!("{}_{}", names[1], i))
                 .collect::<Vec<_>>();
 
-            assert_eq!(expected, tests);
+            assert_eq!(expected.len(), tests.len());
+            for (e, m) in expected.into_iter().zip(tests.into_iter()) {
+                assert_in!(m, e);
+            }
         }
     }
 
@@ -1356,18 +1718,18 @@ mod matrix_cases_should {
 
         let mods = tg.get_modules().names();
 
-        assert_eq!(mods[0], "first_001");
-        assert_eq!(mods[99], "first_100");
+        assert_in!(mods[0], "first_001");
+        assert_in!(mods[99], "first_100");
 
         let mods = tg.get_modules()[0].get_modules().names();
 
-        assert_eq!(mods[0], "second_01");
-        assert_eq!(mods[9], "second_10");
+        assert_in!(mods[0], "second_01");
+        assert_in!(mods[9], "second_10");
 
         let functions = tg.get_modules()[0].get_modules()[1].get_tests().names();
 
-        assert_eq!(functions[0], "third_1");
-        assert_eq!(functions[1], "third_2");
+        assert_in!(functions[0], "third_1");
+        assert_in!(functions[1], "third_2");
     }
 }
 
@@ -1485,7 +1847,8 @@ mod complete_should {
         let attrs = attrs("#[first]#[second(arg)]");
 
         for f in modules[0].get_all_tests() {
-            assert_eq!(attrs, &f.attrs[1..]);
+            let end = f.attrs.len() - 1;
+            assert_eq!(attrs, &f.attrs[1..end]);
         }
         for f in modules[1].get_all_tests() {
             assert_eq!(attrs, &f.attrs[1..3]);
@@ -1497,7 +1860,7 @@ mod complete_should {
         let attrs = attrs("#[third]#[forth(other)]");
 
         for f in modules[1].get_all_tests() {
-            assert_eq!(attrs, &f.attrs[3..]);
+            assert_eq!(attrs, &f.attrs[3..5]);
         }
     }
 }
